import * as React from 'react';
export declare type Action = string;
export declare type BinaryKind = 'arraybuffer' | 'blob';
export interface Option<BK extends BinaryKind> {
    readonly binaryType: BK;
    readonly serialize: Serialize;
    readonly deserialize: Deserialize<BK>;
    readonly heartbeat?: HeartbeatOption;
}
export declare type Packet = string | ArrayBufferLike | Blob | ArrayBufferView;
export declare type Serialize = (...data: any[]) => Packet;
export declare type Deserialize<BK extends BinaryKind> = (packet: string | (BK extends 'arraybuffer' ? ArrayBuffer : Blob)) => void | SerializedPacket;
export interface SerializedPacket {
    readonly action: Action;
    readonly data: any[];
}
export interface HeartbeatOption {
    readonly interval: number;
    readonly action: Action;
    readonly data?: any[];
}
export interface HeartbeatContext {
    readonly latestHeartbeat: null | Date;
    readonly setNextDelay: (nextDelay: number | ((previous: number) => number)) => void;
    readonly setData: (...data: any[]) => void;
}
export interface WebSocketContext {
    readonly removeListener: RemoveListener;
    readonly addListener: AddListener;
    readonly once: Once;
    readonly emit: Emit;
    readonly reconnect: Reconnect;
}
export declare type Handler = (...data: any[]) => void;
export declare type Handle = (action: Action, handler: Handler) => WebSocketContext;
export declare type RemoveListener = Handle;
export declare type AddListener = Handle;
export declare type Once = Handle;
export declare type Emit = (...data: any[]) => WebSocketContext;
export declare type Reconnect = () => void;
export interface Props {
    readonly url: string;
    readonly middleware?: Middleware;
    readonly onOpen?: OnOpen;
    readonly onError?: OnError;
    readonly onClose?: OnClose;
}
export declare type Middleware = (action: string, ...data: any[]) => void;
export declare type OnOpen = (event: Event) => void;
export declare type OnError = (event: Event) => void;
export declare type OnClose = (event: CloseEvent) => void;
export declare const createWebSocket: <BK extends BinaryType>({ binaryType, serialize, deserialize, heartbeat }: Option<BK>) => {
    WebSocketProvider: React.FunctionComponent<Props>;
    WebSocketConsumer: React.ExoticComponent<React.ConsumerProps<WebSocketContext>>;
    useWebSocket: () => WebSocketContext;
    useHeartbeat: () => HeartbeatContext;
};
