"use strict";
exports.__esModule = true;
var React = require("react");
var eventemitter3_1 = require("eventemitter3");
var react_1 = require("react");
exports.createWebSocket = function (_a) {
    var binaryType = _a.binaryType, serialize = _a.serialize, deserialize = _a.deserialize, heartbeat = _a.heartbeat;
    var Context = react_1.createContext({});
    var event = new eventemitter3_1.EventEmitter();
    var pendingQueue = [];
    var useWebSocket = function () { return react_1.useContext(Context); };
    var WebSocketProvider = function (_a) {
        var url = _a.url, middleware = _a.middleware, onOpen = _a.onOpen, _b = _a.onError, onError = _b === void 0 ? function () { } : _b, onClose = _a.onClose, children = _a.children;
        var createWebSocket = function () {
            var nextWebSocket = new WebSocket(url);
            nextWebSocket.binaryType = binaryType;
            nextWebSocket.onerror = onError;
            nextWebSocket.onclose = function () {
                nextWebSocket.onmessage = null;
                nextWebSocket.onerror = null;
                onOpen && nextWebSocket.removeEventListener('open', onOpen);
                onClose && nextWebSocket.removeEventListener('close', onClose);
            };
            onClose && nextWebSocket.addEventListener('close', onClose);
            nextWebSocket.onmessage = function (_a) {
                var packet = _a.data;
                var decodedPacket = deserialize(packet);
                var _b = decodedPacket || {}, action = _b.action, data = _b.data;
                if (!action) {
                    return;
                }
                if (middleware) {
                    middleware.apply(void 0, [action].concat(data));
                }
                event.emit.apply(event, [action].concat(data));
            };
            nextWebSocket.onopen = function () {
                pendingQueue.forEach(function (packet) { return nextWebSocket.send(packet); });
                setCurrentWebSocket(nextWebSocket);
            };
            if (onOpen) {
                nextWebSocket.addEventListener('open', onOpen);
            }
            return nextWebSocket;
        };
        var _c = react_1.useState(createWebSocket), currentWebSocket = _c[0], setCurrentWebSocket = _c[1];
        var isBroadcastable = function () { return currentWebSocket.readyState === currentWebSocket.OPEN; };
        var createContext = function () { return ({
            removeListener: function (action, handler) {
                event.removeListener(action, handler);
                return this;
            },
            addListener: function (action, handler) {
                event.addListener(action, handler);
                return this;
            },
            once: function (action, handler) {
                event.once(action, handler);
                return this;
            },
            emit: function () {
                var data = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    data[_i] = arguments[_i];
                }
                var packet = serialize.apply(void 0, data);
                if (isBroadcastable()) {
                    currentWebSocket.send(packet);
                }
                else {
                    pendingQueue.push(packet);
                }
                return this;
            },
            reconnect: function () {
                if (isBroadcastable()) {
                    currentWebSocket.close();
                }
                setCurrentWebSocket(createWebSocket());
            }
        }); };
        var context = react_1.useMemo(createContext, [currentWebSocket]);
        return (React.createElement(Context.Provider, { value: context },
            React.createElement(HeartbeatProvider, { option: heartbeat }, children)));
    };
    var HeartbeatContext = react_1.createContext([]);
    var useHeartbeat = function () { return react_1.useContext(HeartbeatContext); };
    var HeartbeatProvider = function (_a) {
        var option = _a.option, children = _a.children;
        var websocket = useWebSocket();
        var action = react_1.useState(option && option.action)[0];
        var _b = react_1.useState((option && option.data) || []), setData = _b[1];
        var _c = react_1.useState(null), latestHeartbeat = _c[0], setLatestHeartbeat = _c[1];
        var _d = react_1.useState(option ? option.interval : 0), nextDelay = _d[0], setNextDelay = _d[1];
        var _e = react_1.useState(0), setTimer = _e[1];
        var sendPing = function () {
            setData(function (data) {
                websocket.emit.apply(websocket, [action].concat(data));
                setLatestHeartbeat(new Date());
                return data;
            });
        };
        var sendHeartbeat = function () {
            if (nextDelay) {
                setTimer(window.setInterval(sendPing, nextDelay));
            }
        };
        react_1.useEffect(sendHeartbeat, []);
        var createContext = function () { return ({
            latestHeartbeat: latestHeartbeat,
            setNextDelay: function (nextDelayGetter) {
                setNextDelay(function (previousDelay) {
                    var delay = typeof nextDelayGetter === 'number' ? nextDelayGetter : nextDelayGetter(previousDelay);
                    setTimer(function (timer) {
                        clearTimeout(timer);
                        return window.setInterval(sendPing, delay);
                    });
                    return delay;
                });
            },
            setData: function () {
                var data = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    data[_i] = arguments[_i];
                }
                setData(data);
            }
        }); };
        var context = react_1.useMemo(createContext, [latestHeartbeat]);
        return React.createElement(HeartbeatContext.Provider, { value: context }, children);
    };
    return {
        WebSocketProvider: WebSocketProvider,
        WebSocketConsumer: Context.Consumer,
        useWebSocket: useWebSocket,
        useHeartbeat: useHeartbeat
    };
};
